package ins;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

public class BooleanSet implements Insieme {
	
	final private static int LENGTH = 100;
	
	boolean[] data;

	private int size, modcount=0;

	public BooleanSet() {
		clear();
	}

	/**
	 * Svuota l'insieme
	 */
	public void clear() {
		size = 0;
		data = new boolean[LENGTH];
		modcount++;
	}

	/**
	 * @return il numero di oggetti presenti nell'insieme
	 */
	public int size() {
		return size;
	}
	
	/**
	 * Verifica che l'insieme sia vuoto.
	 * 
	 * @return true se l'insieme e vuoto false altrimenti.
	 */
	public boolean isEmpty() {
		return size == 0;
	}

	/**
	 * Controlla se un intero è presente
	 * 
	 * @return true se l'intero è presente, false altrimenti
	 */
	public boolean contains(int x) {
		if (x < 0 || x > data.length) return false;
		return data[x];
	}

	/**
	 * Inserisce un int nell'insieme
	 * 
	 * @param x
	 *            l'intero da inserire.
	 * @return true se l'intero non era già presente, false altrimenti
	 * @exception IllegalArgumentException se x<0
	 */
	public boolean put(int x) {
		if (x < 0) throw new IllegalArgumentException(" x<0 ");
		if (x >= data.length) increase(x);
		else if (data[x]) return false;
		data[x]=true;
		size++;
		modcount++;
		return true;
	}

	private void increase(int x) {
		int newLength = data.length*2;
		if(x>=newLength)newLength=x+1;
		boolean[] newArray = new boolean[newLength];
		System.arraycopy(data, 0, newArray, 0, data.length);
		data = newArray;
	}

	/**
	 * Rimuove un int nell'insieme.
	 * 
	 * @param x
	 *            l'intero da rimuovere.
	 * @return true se l'intero era presente, false altrimenti
	 */
	public boolean remove(int x) {
		if (x < 0 || x > data.length) return false;
		if (!data[x]) return false;
		data[x] = false;
		size--;
		modcount++;
		return true;
	}
	

	public int nextTrue(int x) {
		if(x>=data.length)return -1;
		if(x<-1) x=-1;
		while(x<data.length && !data[++x]);
		if(x>=data.length)return -1;
		return x;
	}

	/**
	 * Conversione in String 
	 * Restituisce una String con gli elementi dell'insieme 
	 * in ordine crescente racchiusi tra parentesi quadre e separati da virgole
	 * 
	 * @return la stringa
	 */
	public String toString() {
			StringBuffer s = new StringBuffer("[");
			
			for(int j : this) s.append(j + " ");
			
			if (s.length() == 1)
				s.append("]");
			else
				s.setCharAt(s.length() - 1, ']');
			return s.toString();
		}
	
	/**
	 * Restituisce un iteratore per scandire gli elementi dell'insieme in ordine
	 * crescente
	 * 
	 * @return l'iteratore
	 * @exception ConcurrentModificationException
	 *                se si usa next() e l'insieme è stata modificato
	 * @exception UnsupportedOperationException
	 *                se si usa remove()
	 */
	public Iterator<Integer> iterator() {
		return new SetIterator(this);
	}


	class SetIterator implements Iterator<Integer> {

		BooleanSet c;

		/**
		 * numero modifiche 
		 */
		int modcount;
		
		/**
		 * puntatore alla posizione corrente 
		 */
		int pun;
		
		/**
		 * numero oggetti restituiti 
		 */
		int out;

		SetIterator(BooleanSet c) {
			this.c = c;
			this.pun=-1;
			this.out=0;
			this.modcount = c.modcount;
		}

		public boolean hasNext() {
			if (this.modcount != c.modcount) 
				throw new ConcurrentModificationException();
			
			return out<c.size;
		}

		public Integer next() {
			if (this.modcount != c.modcount) 
				throw new ConcurrentModificationException();
  			if (!hasNext()) throw new NoSuchElementException();    
  			
 			pun = c.nextTrue(pun);
			out++;
			return pun;
		}

		public void remove() {
			throw new UnsupportedOperationException();
		}
	}

}
